from ussd.core import UssdHandlerAbstract
from ussd.screens.serializers import NextUssdScreenSerializer
from rest_framework import serializers
import requests
from ussd.tasks import http_task
import json


class HttpScreenConfSerializer(serializers.Serializer):
    method = serializers.ChoiceField(
        ("post", "get", "put", "delete")
    )
    url = serializers.URLField()


class HttpScreenSerializer(NextUssdScreenSerializer):
    session_key = serializers.CharField()
    synchronous = serializers.BooleanField(required=False)
    http_request = HttpScreenConfSerializer()


class HttpScreen(UssdHandlerAbstract):
    screen_type = "http_screen"
    serializer = HttpScreenSerializer

    def render_request_conf(self, data):
        if isinstance(data, str):
            return self._render_text(data)

        elif isinstance(data, list):
            list_data = []
            for i in data:
                list_data.append(self.render_request_conf(i))

            return list_data

        elif isinstance(data, dict):
            dict_data = {}
            for key, value in data.items():
                dict_data.update(
                    {key: self.render_request_conf(value)}
                )
            return dict_data
        else:
            return data

    def handle(self):
        http_request_conf = self.render_request_conf(
            self.screen_content['http_request']
        )
        response_to_save = {}
        if self.screen_content.get('synchronous', False):
            http_task.delay(request_conf=http_request_conf)
        else:
            self.logger.info("sending_request", **http_request_conf)
            response = requests.request(**http_request_conf)
            self.logger.info("response", status_code=response.status_code, content=response.content)
            response_to_save = json.loads(response.content.decode())
        # save response in session
        self.ussd_request.session[self.screen_content['session_key']] = response_to_save

        return self.ussd_request.forward(self.screen_content['next_screen'])



from ussd.core import UssdHandlerAbstract, UssdResponse
from ussd.screens.serializers import UssdContentBaseSerializer


class QuitScreen(UssdHandlerAbstract):
    screen_type = "quit_screen"
    serializer = UssdContentBaseSerializer

    def handle(self):
        return UssdResponse(self.get_text(), status=False)



from ussd.core import UssdView, UssdRequest
from ussd.tests.sample_screen_definition import path
from django.http import HttpResponse


class AfricasTalkingUssdGateway(UssdView):

    def post(self, req):
        ussd_request = UssdRequest(
            phone_number=req.data['phoneNumber'].strip('+'),
            session_id=req.data['sessionId'],
            ussd_input=req.data['text'],
            service_code=req.data['serviceCode'],
            language=req.data.get('language', 'en')
        )

        return ussd_request

    def get_customer_journey_conf(self, request):
        if request.data.get('customer_journey_conf'):
            return path + '/' + request.data.get('customer_journey_conf')
        return path + "/valid_input_screen_conf.yml"

    def get_customer_journey_namespace(self, request):
        if request.data.get('customer_journey_conf'):
            return request.data['customer_journey_conf'].replace(
                '.yml', ''
            )
        return "AfricasTalkingUssdGateway"

    def ussd_response_handler(self, ussd_response):
        if self.request.data.get('serviceCode') == 'test':
            return super(AfricasTalkingUssdGateway, self).\
                ussd_response_handler(ussd_response)
        if ussd_response.status:
            res = 'CON' + ' ' + str(ussd_response)
            response = HttpResponse(res)
        else:
            res = 'END' + ' ' + str(ussd_response)
            response = HttpResponse(res)
        return response
